"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var FactoryTemplate_1 = require("./FactoryTemplate");
var FactoryMate = (function () {
    function FactoryMate() {
    }
    FactoryMate.define = function (cns, initFunction) {
        FactoryMate.defineWithName(cns, cns.name, initFunction);
    };
    FactoryMate.defineWithName = function (cns, alias, initFunction) {
        FactoryMate.storeTemplate(alias, new FactoryTemplate_1.FactoryTemplate(cns, initFunction));
    };
    FactoryMate.build = function (itemName, overrideFn) {
        var clazz = this.locateConstructor(itemName);
        var clazzInstance = new clazz.classConstructor();
        var builtObject = clazz.initializationFunction(clazzInstance);
        if (overrideFn) {
            overrideFn(builtObject);
        }
        return builtObject;
    };
    FactoryMate.buildMany = function (itemName, numberToBuild, overrideFn) {
        if (numberToBuild === void 0) { numberToBuild = 1; }
        if (numberToBuild < 1) {
            throw new Error("Number of Items to Build Must Be 1 or Higher. Received " + numberToBuild);
        }
        var createdClasses = new Array();
        for (var i = 0; i < numberToBuild; i++) {
            createdClasses.push(FactoryMate.build(itemName, overrideFn));
        }
        return createdClasses;
    };
    FactoryMate.storeTemplate = function (alias, template) {
        if (FactoryMate.definedConstructors.get(alias)) {
            throw new Error("A template with alias '" + alias + "' has been registered already.");
        }
        FactoryMate.definedConstructors.set(alias, template);
    };
    FactoryMate.locateConstructor = function (objectName) {
        var clazz = FactoryMate.definedConstructors.get(objectName);
        if (clazz === undefined) {
            throw new Error("Class with name " + objectName + " is not registered to FactoryMate.");
        }
        return clazz;
    };
    FactoryMate.definedConstructors = new Map();
    return FactoryMate;
}());
exports.FactoryMate = FactoryMate;
